/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;
import de.ixxat.jni.CPointer;

/**
 * This value class represents the CAN Capabilities. The CAN Capabilities
 * can be retrieved by calling {@link ICanSocket#GetCapabilities()}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class CanCapabilities
{
  // CAN Controller Types
  /** Unknown CAN controller Type */
  public static final short CAN_CTRL_UNKNOWN    = 0;
  /** Intel 82527 */
  public static final short CAN_CTRL_82527      = 1;
  /** Intel 82C200 */
  public static final short CAN_CTRL_82C200     = 2;
  /** Intel 82C90 */
  public static final short CAN_CTRL_82C90      = 3;
  /** Intel 82C92 */
  public static final short CAN_CTRL_82C92      = 4;
  /** Philips SJA 1000 */
  public static final short CAN_CTRL_SJA1000    = 5;
  /** Infinion 82C900 (TwinCAN) */
  public static final short CAN_CTRL_82C900     = 6;
  /** Motorola TOUCAN */
  public static final short CAN_CTRL_TOUCAN     = 7;

  // supported features
  /** 11 OR 29 bit (exclusive) */
  public static final int CAN_FEATURE_STDOREXT  = 0x0001;
  /** 11 AND 29 bit (simultaneous) */
  public static final int CAN_FEATURE_STDANDEXT = 0x0002;
  /** reception of remote frames */
  public static final int CAN_FEATURE_RMTFRAME  = 0x0004;
  /** reception of error frames */
  public static final int CAN_FEATURE_ERRFRAME  = 0x0008;
  /** bus load measurement */
  public static final int CAN_FEATURE_BUSLOAD   = 0x0010;
  /** exact message filter */
  public static final int CAN_FEATURE_IDFILTER  = 0x0020;
  /** listen only mode */
  public static final int CAN_FEATURE_LISTONLY  = 0x0040;
  /** cyclic message scheduler */
  public static final int CAN_FEATURE_SCHEDULER = 0x0080;
  /** error frame generation */
  public static final int CAN_FEATURE_GENERRFRM = 0x0100;
  /** delayed message transmitter */
  public static final int CAN_FEATURE_DELAYEDTX = 0x0200;

  // bus coupling types
  /** undefined */
  public static final short CAN_BUSC_UNDEFINED  = 0x0000;
  /** low speed coupling */
  public static final short CAN_BUSC_LOWSPEED   = 0x0001;
  /** high speed coupling */
  public static final short CAN_BUSC_HIGHSPEED  = 0x0002;

  /** Type of CAN controller 
   * @see #CAN_CTRL_UNKNOWN */
  public short  m_wCtrlType     = 0;     
  /** Type of Bus coupling 
   * @see #CAN_BUSC_UNDEFINED */
  public short  m_wBusCoupling  = 0;
  /** supported features 
   * @see #CAN_FEATURE_STDOREXT */
  public int    m_dwFeatures    = 0;
  /** clock frequency of the primary counter in Hz */
  public int    m_dwClockFreq   = 0; 
  /** divisor for the message time stamp counter */
  public int    m_dwTscDivisor  = 0;
  /** divisor for the cyclic message scheduler */
  public int    m_dwCmsDivisor  = 0;
  /** maximum tick count value of the cyclic message scheduler */
  public int    m_dwCmsMaxTicks = 0;
  /** divisor for the delayed message transmitter */
  public int    m_dwDtxDivisor  = 0;
  /** maximum tick count value of the delayed message transmitter */
  public int    m_dwDtxMaxTicks = 0;

  
  /**
   * Creates a new CAN Capabilities object.
   * @throws Throwable
   */
  protected CanCapabilities()
  {
  }
  
  /**
   * Creates a new CAN Capabilities object.
   * 
   * @param pCanCapabilities
   *          CPointer to native memory which contains the CANCAPABILITIES
   *          structure.
   * @throws Throwable
   */
  protected CanCapabilities(CPointer pCanCapabilities) throws Throwable
  {
    fromCPointer(pCanCapabilities);
  }
  
  /**
   * Updates a existing CAN Capabilities object.
   * 
   * @param pCanCapabilities
   *          CPointer to native memory which contains the CANCAPABILITIES
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pCanCapabilities) throws Throwable
  {
    // Update CAN Capabilities
    m_wCtrlType     = pCanCapabilities.getShort(0);     
    m_wBusCoupling  = pCanCapabilities.getShort(2);
    m_dwFeatures    = pCanCapabilities.getInt(4);
    m_dwClockFreq   = pCanCapabilities.getInt(8); 
    m_dwTscDivisor  = pCanCapabilities.getInt(12);
    m_dwCmsDivisor  = pCanCapabilities.getInt(16);
    m_dwCmsMaxTicks = pCanCapabilities.getInt(20);
    m_dwDtxDivisor  = pCanCapabilities.getInt(24);
    m_dwDtxMaxTicks = pCanCapabilities.getInt(28);
    
  }

  /**
   * Returns the CAN Capabilities as string.
   * @return The CAN Capabilities as string.
   */
  public String toString()
  {
    String strCapabilities = "";
    
    strCapabilities += "Controller Type: " + m_wCtrlType + ", ";
    strCapabilities += "Bus Coupling: " + m_wBusCoupling + ", ";
    strCapabilities += "Features: 0x" + ZeroHex.toZeroHexString(m_dwFeatures) + ", ";
    strCapabilities += "ClockFreq: " + m_dwClockFreq + ", ";
    strCapabilities += "TscDivisor: " + m_dwTscDivisor + ", ";
    strCapabilities += "CmsDivisor: " + m_dwCmsDivisor + ", ";
    strCapabilities += "CmsMaxTicks: " + m_dwCmsMaxTicks + ", ";
    strCapabilities += "DtxDivisor: " + m_dwDtxDivisor + ", ";
    strCapabilities += "DtxMaxTicks: " + m_dwDtxMaxTicks;
    
    return strCapabilities;
  }

}
