/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;

/**
 * This class implements the interface {@link ICanChannel} which represents a
 * CAN communication channel and is used to receive and transmit CAN messages.
 * 
 * @see ICanChannel
 * @author Michael Ummenhofer
 * 
 */
public class CanChannel extends CInterface implements ICanChannel
{
  private static final int Initialize = 3;
  private static final int Activate   = 4;
  private static final int Deactivate = 5;
  private static final int GetReader  = 6;
  private static final int GetWriter  = 7;
  private static final int GetStatus  = 8;

  // Interface functions
  private CFunction m_fpInitialize  = null;
  private CFunction m_fpActivate    = null;
  private CFunction m_fpDeactivate  = null;
  private CFunction m_fpGetReader   = null;
  private CFunction m_fpGetWriter   = null;
  private CFunction m_fpGetStatus   = null;

  // Interface Pointer
  private CPointer m_ICanChannel = null; // Interface Pointer itself (pThis)
  
  /** CAN Channel Status */
  private CanChannelStatus m_oCanChannelStatus = new CanChannelStatus();

  /**
   * constructor
   * 
   * @param pICanChannel
   * @throws Throwable
   */
  protected CanChannel(CMalloc pICanChannel) throws Throwable
  {
    super(pICanChannel);

    try
    {
      // Retrieve Interface Pointer Address
      m_ICanChannel = super.m_Interface;

      // Retrieve Interface Methods
      m_fpInitialize  = new CFunction(m_ICanChannel, Initialize,  "JNI");
      m_fpActivate    = new CFunction(m_ICanChannel, Activate,    "JNI");
      m_fpDeactivate  = new CFunction(m_ICanChannel, Deactivate,  "JNI");
      m_fpGetReader   = new CFunction(m_ICanChannel, GetReader,   "JNI");
      m_fpGetWriter   = new CFunction(m_ICanChannel, GetWriter,   "JNI");
      m_fpGetStatus   = new CFunction(m_ICanChannel, GetStatus,   "JNI");
    } 
	catch (Exception oException)
    {
      // Release all Pointers without freeing memory
      m_ICanChannel   = null;
      m_fpInitialize  = null;
      m_fpActivate    = null;
      m_fpDeactivate  = null;
      m_fpGetReader   = null;
      m_fpGetWriter   = null;
      m_fpGetStatus   = null;
      
      throw oException;
    }
  }

  /**
   * Clean up Object
   * 
   * @throws Throwable
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    } 
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_ICanChannel   = null;
      m_fpInitialize  = null;
      m_fpActivate    = null;
      m_fpDeactivate  = null;
      m_fpGetReader   = null;
      m_fpGetWriter   = null;
      m_fpGetStatus   = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }

  /**
   * @see ICanChannel#Initialize(short, short)
   * @throws Throwable
   */
  public void Initialize(short wRxSize, short wTxSize) throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpInitialize.callInt(new Object[] { m_ICanChannel, new Integer(wRxSize), new Integer(wTxSize) });
      if (hResult != 0)
      {
        throw new VciException("ICanChannel->Initialize failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    } 
	finally
    {
    }
  }

  /**
   * @see ICanChannel#Activate()
   * @throws Throwable
   */
  public void Activate() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpActivate.callInt(new Object[] { m_ICanChannel });
      if (hResult != 0)
      {
        throw new VciException("ICanChannel->Activate failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    } 
	finally
    {
    }
  }

  /**
   * @see ICanChannel#Deactivate()
   * @throws Throwable
   */
  public void Deactivate() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpDeactivate.callInt(new Object[] { m_ICanChannel });
      if (hResult != 0)
      {
        throw new VciException("ICanChannel->Deactivate failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    } 
	finally
    {
    }
  }

  /**
   * @see ICanChannel#GetMessageReader()
   * @return {@link ICanMessageReader}
   * @throws Throwable
   */
  public ICanMessageReader GetMessageReader() throws Throwable
  {
    CanMessageReader oCanMessageReader = null;
    int hResult = 0;

    // Create CanMessageReader Interface Memory
    CMalloc pICanMessageReader = new CMalloc(8);
    try
    {
      pICanMessageReader.setLong(0, 0);

      // Retrieve IFifoReader Interface
      hResult = m_fpGetReader.callInt(new Object[] { m_ICanChannel, pICanMessageReader });
      if (hResult != 0)
      {
        throw new VciException("ICanChannel->GetReader failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
	  else
      {
        // Create new CanMessageReader Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the CanMessageReader Object
        oCanMessageReader = new CanMessageReader(pICanMessageReader);
      }
    } 
	catch (Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pICanMessageReader.free();
      throw oException;
    } 
	finally
    {
    }

    return oCanMessageReader;
  }

  /**
   * @see ICanChannel#GetMessageWriter()
   * @return {@link ICanMessageWriter}
   * @throws Throwable
   */
  public ICanMessageWriter GetMessageWriter() throws Throwable
  {
    CanMessageWriter oCanMessageWriter = null;
    int hResult = 0;

    // Create CanMessageWriter Interface Memory
    CMalloc pICanMessageWriter = new CMalloc(8);
    try
    {
      pICanMessageWriter.setLong(0, 0);

      // Retrieve IFifoWriter Interface
      hResult = m_fpGetWriter.callInt(new Object[] { m_ICanChannel, pICanMessageWriter });
      if (hResult != 0)
      {
        throw new VciException("ICanChannel->GetWriter failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        // Create new CanMessageWriter Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the CanMessageReader Object
        oCanMessageWriter = new CanMessageWriter(pICanMessageWriter);
      }
    } 
    catch (Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pICanMessageWriter.free();
      throw oException;
    } 
    finally
    {
    }

    return oCanMessageWriter;
  }

  /**
   * @see ICanChannel#GetStatus()
   * @throws Throwable
   */
  public CanChannelStatus GetStatus() throws Throwable
  {
    int hResult = 0;

    // Create CANCHANSTATUS Memory
    CMalloc pCanChanStatus = new CMalloc(18);
    try
    {
      hResult = m_fpGetStatus.callInt(new Object[] { m_ICanChannel, pCanChanStatus });
      if (hResult != 0)
      {
        throw new VciException("ICanChannel->GetStatus failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oCanChannelStatus.fromCPointer(pCanChanStatus);
      }
    } 
    finally
    {
      pCanChanStatus.free();
    }
    
    return m_oCanChannelStatus;
  }

}
