/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;
import de.ixxat.jni.CPointer;

/**
 * This value class represents a cyclic CAN TX message object. Cyclic CAN
 * messages can be transmitted via the the message scheduler
 * {@link ICanScheduler}.
 * 
 * @author Michael Ummenhofer
 * 
 */

public class CanCyclicTxMessage
{
  /** no increment */
  public static final byte CAN_CTXMSG_INC_NO       =  0x00;
  /** increment CAN identifier */
  public static final byte CAN_CTXMSG_INC_ID       =  0x01;
  /** increment 8-bit data field */
  public static final byte CAN_CTXMSG_INC_8        =  0x02;
  /** increment 16-bit data field */
  public static final byte CAN_CTXMSG_INC_16       =  0x03;

  /**
   * Cycle time for the message in ticks
   */
  public short m_wCycleTime = 0;

  /**
   * auto increment mode see {@link #CAN_CTXMSG_INC_NO}
   */
  public byte m_bIncrMode  = 0;
  
  /**
   * index of the byte within {@link #m_abData} to increment
   */
  public byte m_bByteIndex = 0; 

  /**
   * Sets the CAN identifier of this CAN message.
   */
  public int m_dwIdentifier = 0;

  /**
   * Sets the data length of this CAN message.
   */
  public byte m_bDataLength = 0;

  /**
   * Sets a single data byte at the specified index from this CAN
   * message. 
   */
  public byte[] m_abData = new byte[8];

  /**
   * Gets or sets a value indicating whether this message has extended frame
   * format (29-bit id).
   * <p>
   * For {@link #toString()}: <code>"E"</code> means extended frame format 
   */
  public boolean m_fExtendedFrameFormat = false;

  /**
   * Gets or sets a value indicating whether this message is the result of a
   * self reception request.
   * <p>
   * For {@link #toString()}: <code>"S"</code> means self reception message 
   */
  public boolean m_fSelfReception = false;

  /**
   * Gets or sets a value indicating whether this message is a remote
   * transmission request.
   * <p>
   * For {@link #toString()}: <code>"R"</code> means remote transmission request 
   */
  public boolean m_fRemoteTransmissionRequest = false;

  /**
   * Constructs a new empty Cyclic CAN TX Message Object.
   */
  public CanCyclicTxMessage()
  {
  }


  protected void finalize() throws Throwable
  {
    try
    {
    } finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }

  /**
   * Updates the given CPointer memory structure with the current CAN cyclic TX
   * message contents.
   * 
   * @param pMsg
   *          CPointer Object which holds a C-Code Memory location with a
   *          CANCYCLICTXMSG structure.
   * @throws Throwable
   */
  protected void toCPointer(CPointer pMsg) throws Throwable
  {
    byte bType      = 0; // CAN_MSGTYPE_DATA
    byte bReserved  = 0; // Always 0
    byte bFlags     = 0;
    byte bAccept    = 0; // Always 0

    // Cycle time for the message in ticks
    pMsg.setShort(0, m_wCycleTime);

    // auto increment mode 
    pMsg.setByte(2, m_bIncrMode);
    
    // index of the byte within abData[] to increment
    pMsg.setByte(3, m_bByteIndex);

    // Set Can Message Identifier
    pMsg.setInt(4, m_dwIdentifier);

    // data frame
    bType = (byte) CanMessage.CAN_MSGTYPE_DATA;

    // data length code
    bFlags |= (m_bDataLength & (byte) CanMessage.CAN_MSGFLAGS_DLC);

    // self reception
    if (m_fSelfReception)
      bFlags |= (byte) CanMessage.CAN_MSGFLAGS_SRR;

    // remote transmission request
    if (m_fRemoteTransmissionRequest)
      bFlags |= (byte) CanMessage.CAN_MSGFLAGS_RTR;

    // Extended Frame Format
    if (m_fExtendedFrameFormat)
      bFlags |= (byte) CanMessage.CAN_MSGFLAGS_EXT;

    // Set CAN Message Info
    pMsg.setByte(8, bType);
    pMsg.setByte(9, bReserved);
    pMsg.setByte(10, bFlags);
    pMsg.setByte(11, bAccept);

    // Set CAN Message Data
    pMsg.copyIn(12, m_abData, 0, m_bDataLength);
  }

  /**
   * Returns a string of the current cyclic TX message.
   * @return String
   */
  public String toString()
  {
    String strMsg = "";
    String strFlags = "";

    // RTR Message?
    if (m_fRemoteTransmissionRequest)
      strFlags += "R";
    else
      strFlags += " ";

    // Self Reception Message?
    if (m_fSelfReception)
      strFlags += "S";
    else
      strFlags += " ";

    // Extended Frame Format?
    if (m_fExtendedFrameFormat)
      strFlags += "E";
    else
      strFlags += " ";

    // Build Message
    strMsg += "Cycletime: " + m_wCycleTime + ", " +
              "Increment Mode: 0x" + ZeroHex.toZeroHexString(m_bIncrMode) +  ", " +
              "Increment Index: " + m_bByteIndex +  ", " +
              " Flags: " + strFlags + ", " +
              " ID: 0x" + ZeroHex.toZeroHexString(m_dwIdentifier);

    // Data available?
    if ((m_bDataLength > 0) && !m_fRemoteTransmissionRequest)
    {
      strMsg += " Data:";
      for (byte bIndex = 0; bIndex < m_bDataLength; bIndex++)
      {
        strMsg += " 0x" + ZeroHex.toZeroHexString(m_abData[bIndex]);
      } // end for
    } // end if
    else if (m_fRemoteTransmissionRequest)
    {
      strMsg += " DLC = " + m_bDataLength;
    }

    return strMsg;
  }
}
