/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;
import de.ixxat.vci3.bal.*;

/**
 * This class implements the interface {@link ICanSocket} which represents a CAN
 * controller properties and capabilities container.
 * 
 * @see ICanSocket
 * @author Michael Ummenhofer
 * 
 */
public class CanSocket extends CInterface implements ICanSocket
{
  private static final int GetSocketInfo  = 3;
  private static final int GetCapabilities= 4;
  private static final int GetLineStatus  = 5;
  private static final int CreateChannel  = 6;

  // Interface functions
  private CFunction m_fpGetSocketInfo   = null;
  private CFunction m_fpGetCapabilities = null;
  private CFunction m_fpGetLineStatus   = null;
  private CFunction m_fpCreateChannel   = null;

  // Interface Pointer
  private CPointer  m_ICanSocket        = null; //Interface Pointer itself (pThis)
  
  /** BAL Socket Info */
  private BalSocketInfo m_oBalSocketInfo      = new BalSocketInfo();
  
  /** CAN Capabilities */
  private CanCapabilities m_oCanCapabilities  = new CanCapabilities();

  /** CAN Line Status */
  private CanLineStatus m_oCanLineStatus      = new CanLineStatus();

  //TODO: Is there an other way to create this object -> "protected"
  /**
   * Constructor
   * 
   * @param pICanSocket
   * @throws Throwable
   */
  public CanSocket(CMalloc pICanSocket) throws Throwable
  {
    super(pICanSocket);
    
    try
    {
      // Retrieve Interface Pointer Address
      m_ICanSocket = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpGetSocketInfo   = new CFunction(m_ICanSocket, GetSocketInfo,  "JNI");
      m_fpGetCapabilities = new CFunction(m_ICanSocket, GetCapabilities,"JNI");
      m_fpGetLineStatus   = new CFunction(m_ICanSocket, GetLineStatus,  "JNI");
      m_fpCreateChannel   = new CFunction(m_ICanSocket, CreateChannel,  "JNI");
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_ICanSocket        = null;
      m_fpGetSocketInfo   = null;
      m_fpGetCapabilities = null;
      m_fpGetLineStatus   = null;
      m_fpCreateChannel   = null;

      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_ICanSocket        = null;
      m_fpGetSocketInfo   = null;
      m_fpGetCapabilities = null;
      m_fpGetLineStatus   = null;
      m_fpCreateChannel   = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
  
  /**
   * @see ICanSocket#GetCapabilities()
   * @throws Throwable
   */
  public CanCapabilities GetCapabilities() throws Throwable
  {
    int hResult = 0;

    // Create CANCAPABILITIES Memory
    CMalloc pCanCapabilities = new CMalloc(32);
    try
    {
      hResult = m_fpGetCapabilities.callInt(new Object[] { m_ICanSocket, pCanCapabilities });
      if (hResult != 0)
      {
        throw new VciException("ICanSocket->GetCapabilities failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oCanCapabilities.fromCPointer(pCanCapabilities);
      }
    } 
    finally
    {
      pCanCapabilities.free();
    }
    
    return m_oCanCapabilities;
  }
  
  /**
   * @see ICanSocket#GetSocketInfo()
   * @throws Throwable
   */
  public BalSocketInfo GetSocketInfo() throws Throwable
  {
    int hResult = 0;

    // Create BALSOCKETINFO Memory
    CMalloc pBalSocketInfo = new CMalloc(12);
    try
    {
      hResult = m_fpGetSocketInfo.callInt(new Object[] { m_ICanSocket, pBalSocketInfo });
      if (hResult != 0)
      {
        throw new VciException("ICanSocket->GetSocketInfo failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oBalSocketInfo.fromCPointer(pBalSocketInfo);
      }
    } 
    finally
    {
      pBalSocketInfo.free();
    }
    
    return m_oBalSocketInfo;
  }

  /**
   * @see ICanSocket#GetLineStatus()
   * @throws Throwable
   */
  public CanLineStatus GetLineStatus() throws Throwable
  {
    int hResult = 0;

    // Create CANLINESTATUS Memory
    CMalloc pCanLineStatus = new CMalloc(8);
    try
    {
      hResult = m_fpGetLineStatus.callInt(new Object[] { m_ICanSocket, pCanLineStatus });
      if (hResult != 0)
      {
        throw new VciException("ICanSocket->GetLineStatus failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oCanLineStatus.fromCPointer(pCanLineStatus);
      }
    } 
    finally
    {
      pCanLineStatus.free();
    }
    
    return m_oCanLineStatus;
  }

  /**
   * @see ICanSocket#CreateChannel(boolean)
   * @throws Throwable
   */
  public ICanChannel CreateChannel(boolean fExclusive) throws Throwable
  {
    CanChannel oCanChannel = null;
    int        hResult     = 0;

    // Create CanChannel Interface Memory
    CMalloc pICanChannel = new CMalloc(8);
    try
    {
      pICanChannel.setLong(0, 0);
      int nExclusive = 0;
      
      // Exclusive Channel?
      if(fExclusive)
        nExclusive = 1;

      // Retrieve ICanChannel Interface
      hResult = m_fpCreateChannel.callInt(new Object[]{ m_ICanSocket, new Integer(nExclusive), pICanChannel});
      if(hResult != 0)
      {
        throw new VciException("ICanSocket->CreateChannel failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        // Create new CanChannel Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the CanChannel Object
        oCanChannel = new CanChannel(pICanChannel);
      }
    }
    catch(Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pICanChannel.free();
      throw oException;
    }
    finally
    {
    }

    return oCanChannel;
  }

}
