/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.ZeroHex;
import de.ixxat.jni.CPointer;

/**
 * This value class represents a LIN message object. LIN messages can be
 * received via the message reader {@link ILinMessageReader} and be transmitted
 * via the function {@link ILinControl#WriteMessage(boolean, LinMessage)}. 
 * 
 * @author Michael Ummenhofer
 * 
 */

public class LinMessage
{
  /** data frame */
  protected static final short LIN_MSGTYPE_DATA     = 0;
  /** info frame */
  protected static final short LIN_MSGTYPE_INFO     = 1;
  /** error frame */
  protected static final short LIN_MSGTYPE_ERROR    = 2;
  /** status frame */
  protected static final short LIN_MSGTYPE_STATUS   = 3;
  /** wakeup frame */
  protected static final short LIN_MSGTYPE_WAKEUP   = 4;
  /** goto sleep frame */
  protected static final short LIN_MSGTYPE_SLEEP    = 5;
  /** timer overrun */
  protected static final short LIN_MSGTYPE_TMOVR    = 6;

  /** enhanced checksum (LIN 2.0) */
  protected static final short LIN_MSGFLAGS_ECS = 0x01;
  /** sender of response */
  protected static final short LIN_MSGFLAGS_SOR = 0x02;
  /** possible data overrun */
  protected static final short LIN_MSGFLAGS_OVR = 0x04;
  /** ID only */
  protected static final short LIN_MSGFLAGS_IDO = 0x08;

  /**
   * With receive messages, this field contains the relative reception time of
   * the message in ticks. The resolution of a tick can be calculated from the
   * properties {@link LinCapabilities#m_dwClockFreq} and
   * {@link LinCapabilities#m_dwTscDivisor}. in accordance with the following
   * in accordance with the following formula:
   * <p>
   * <blockquote><code> 
   * Resolution [s] = TimeStampCounterDivisor / ClockFrequency
   * </code></blockquote>
   */
  public int m_dwTimestamp = 0;

  /**
   * Gets or sets the LIN identifier of this LIN message.
   */
  public byte m_bIdentifier = 0;

  /**
   * Gets or sets the data length of this LIN message.
   */
  public byte m_bDataLength = 0;

  /**
   * Gets or sets a single data byte at the specified index from this LIN
   * message. 
   */
  public byte[] m_abData = new byte[8];

  /**
   * Gets or sets a value indicating whether this message has enhanced checksum
   * format (checksum according to LIN Spec. 1.3 or 2.0).
   * <p>
   * For {@link LinMessage#toString()}: <code>"E"</code> means enhanced
   * checksum format
   */
  public boolean m_fEnhancedChecksum = false;

  /**
   * Gets a value indicating whether this message is the result of a self sent
   * response.
   * <p>
   * For {@link LinMessage#toString()}: <code>"S"</code> means sender of
   * response
   */
  public boolean m_fSenderOfResponse = false;

  /**
   * Gets a value indicating whether this message was the last message which
   * could be placed in the receive FIFO, before this overflows.
   * <p>
   * For {@link LinMessage#toString()}: <code>"O"</code> means possible overrun 
   */
  public boolean m_fPossibleOverrun = false;

  /**
   * Gets or sets a value indicating whether this message is a ID only message
   * (without trailing data)
   * <p>
   * For {@link LinMessage#toString()}: <code>"I"</code> means ID only
   * message
   */
  public boolean m_fIdOnly = false;

  /**
   * Constructs a new empty LIN Message Object.
   */
  public LinMessage()
  {
  }

  /**
   * Constructs a new LIN Message Object using the LIN Object given as CPointer.
   * 
   * @param pMsg
   *          CPointer Object which holds a C-Code Memory location with a LINMSG
   *          structure.
   * @throws Throwable
   */
  protected LinMessage(CPointer pMsg) throws Throwable
  {
    fromCPointer(pMsg);
  }

  protected void finalize() throws Throwable
  {
    try
    {
    } finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }

  /**
   * Update a existing LIN Message Object using the LIN Object given as
   * CPointer.
   * 
   * @param pMsg
   *          CPointer Object which holds a C-Code Memory location with a LINMSG
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pMsg) throws Throwable
  {
    byte bPid   = 0;
    byte bType  = 0;
    byte bDlen  = 0;
    byte bFlags = 0;

    // Get LIN Message Info
    bPid    = pMsg.getByte(4);
    bType   = pMsg.getByte(5);
    bDlen   = pMsg.getByte(6);
    bFlags  = pMsg.getByte(7);

    // TODO: Momentarily data frames supported, only!
    // Is data frame?
    // all other messages will throw an exception which leads to a removal of
    // the message
    if (bType != (byte) LIN_MSGTYPE_DATA)
      throw new IllegalArgumentException("Message is not LIN_MSGTYPE_DATA");

    // Get TimeStamp
    m_dwTimestamp = pMsg.getInt(0);

    // Get LIN Message Identifier
    m_bIdentifier = (byte) (0x3F & bPid); // Mask out Parity BITs

    // data length code
    m_bDataLength = bDlen;
   
    // enhanced checksum (LIN 2.0)
    if ((byte) (bFlags & (byte) LIN_MSGFLAGS_ECS) != 0)
      m_fEnhancedChecksum = true;
    else
      m_fEnhancedChecksum = false;

    // sender of response
    if ((byte) (bFlags & (byte) LIN_MSGFLAGS_SOR) != 0)
      m_fSenderOfResponse = true;
    else
      m_fSenderOfResponse = false;

    // data overrun flag
    if ((byte) (bFlags & (byte) LIN_MSGFLAGS_OVR) != 0)
      m_fPossibleOverrun = true;
    else
      m_fPossibleOverrun = false;

    // ID only 
    if ((byte) (bFlags & (byte) LIN_MSGFLAGS_IDO) != 0)
      m_fIdOnly = true;
    else
      m_fIdOnly = false;


    // Get LIN Message Data
    pMsg.copyOut(8, m_abData, 0, m_bDataLength);
  }

  /**
   * Updates the given CPointer memory structure with the current LinMessage
   * contents.
   * 
   * @param pMsg
   *          CPointer Object which holds a C-Code Memory location with a LINMSG
   *          structure.
   * @throws Throwable
   */
  protected void toCPointer(CPointer pMsg) throws Throwable
  {
    byte bPid   = 0; 
    byte bType  = 0; //LIN_MSGTYPE_DATA
    byte bDlen  = 0; //LIN Message Length
    byte bFlags = 0;

    // TimeStamp now serves as message delay
    // set to zero to send the message immediately
    pMsg.setInt(0, m_dwTimestamp);

    // Set LIN Message Identifier
    bPid = (byte)(0x3F & m_bIdentifier); // Mask out Parity BITs

    // data frame
    bType = (byte) LIN_MSGTYPE_DATA;

    // data length code
    bDlen = m_bDataLength;

    // enhanced checksum (LIN 2.0)
    if (m_fEnhancedChecksum)
      bFlags |= (byte) LIN_MSGFLAGS_ECS;

    // ID only
    if (m_fIdOnly)
      bFlags |= (byte) LIN_MSGFLAGS_IDO;

    // Set LIN Message Info
    pMsg.setByte(4, bPid);
    pMsg.setByte(5, bType);
    pMsg.setByte(6, bDlen);
    pMsg.setByte(7, bFlags);

    // Set LIN Message Data
    pMsg.copyIn(8, m_abData, 0, m_bDataLength);
  }

  /**
   * Returns a string of the current message.
   * @return String
   */
  public String toString()
  {
    String strMsg = "";
    String strFlags = "";

    // enhanced checksum (LIN 2.0)?
    if (m_fEnhancedChecksum)
      strFlags += "E";
    else
      strFlags += " ";

    // sender of response?
    if (m_fSenderOfResponse)
      strFlags += "S";
    else
      strFlags += " ";

    // Possible overrun?
    if (m_fPossibleOverrun)
      strFlags += "O";
    else
      strFlags += " ";

    // ID only ?
    if (m_fIdOnly)
      strFlags += "I";
    else
      strFlags += " ";


    // Build Message
    strMsg += "Timestamp: " + m_dwTimestamp + " Flags: " + strFlags + " ID: 0x"
        + ZeroHex.toZeroHexString(m_bIdentifier);

    // Data available?
    if ((m_bDataLength > 0) && (!m_fIdOnly)) 
    {
      strMsg += " Data:";
      for (byte bIndex = 0; bIndex < m_bDataLength; bIndex++)
      {
        strMsg += " 0x" + ZeroHex.toZeroHexString(m_abData[bIndex]);
      } // end for
    } // end if

    return strMsg;
  }
}
